//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

struct linkinfo {
	void* targetlist; //linked list of null-terminated target addresses (possibly unresolved hosts BUT this is unimplemented); constructed to a longer list when merging entries
	void* passwordlist; //linked list of null-terminated password couples ("<sendpw> <receivepw>"); constructed to a longer list when merging entries
	char* bind; //null-terminated bind interface string
	char* password;
	char* recvpassword;
	char* protocol;
	int port;
	int iptype;
};
void* linkinfoblocks = NULL; 
struct linkinfo* currentlinkinfo = NULL;

void linkinfo_ClearItemData(void* item) {
	struct linkinfo* i = linkedlist_GetData(item);
	if (i->targetlist != NULL) {linkedlist_Free((void**)&i->targetlist);}
	if (i->passwordlist != NULL) {linkedlist_Free((void**)&i->passwordlist);}
	if (i->password != NULL) {free(i->password);}
	if (i->bind != NULL) {free(i->bind);}
	if (i->protocol != NULL) {free(i->protocol);}
}
void linkinfo_ClearList() {
	void* item = currentlinkinfo;
	while (item != NULL) {
		linkinfo_ClearItemData(item);
		linkedlist_RemoveItem(&linkinfoblocks,item);
		item = currentlinkinfo;
	}
}

void linkinfo_NewBlock() { /* start a new linking information block that keeps config info on a single link */
	struct linkinfo l;
	memset(&l,0,sizeof(l));
	void* r = linkedlist_Append(&linkinfoblocks,&l,sizeof(l));
	if (r != NULL) {
		currentlinkinfo = linkedlist_GetData(r);
	}
}
void linkinfo_SetProtocol(const char* protocol) {
	if (currentlinkinfo == NULL) {return;}
	currentlinkinfo->protocol = malloc(strlen(protocol)+1);
	if (currentlinkinfo->protocol != NULL) {strcpy(currentlinkinfo->protocol,protocol);}
}
void linkinfo_SetPassword(const char* pass) {
	if (currentlinkinfo == NULL) {return;}
	currentlinkinfo->password = malloc(strlen(pass)+1);
	if (currentlinkinfo->password != NULL) {strcpy(currentlinkinfo->password,pass);}
}
void linkinfo_SetRecvPassword(const char* pass) {
	if (currentlinkinfo == NULL) {return;}
	currentlinkinfo->recvpassword = malloc(strlen(pass)+1);
	if (currentlinkinfo->recvpassword != NULL) {strcpy(currentlinkinfo->recvpassword,pass);}
}
int linkinfo_IsProtocolSet() {
	if (currentlinkinfo == NULL) {return -1;}
	if (currentlinkinfo->protocol != NULL) {return 0;}
	return -1;
}
int linkinfo_IsPasswordSet() {
	if (currentlinkinfo == NULL) {return -1;}
	if (currentlinkinfo->password != NULL) {return 0;}
	return -1;
}
int linkinfo_IsRecvPasswordSet() {
	if (currentlinkinfo == NULL) {return -1;}
	if (currentlinkinfo->recvpassword != NULL) {return 0;}
	return -1;
}
int linkinfo_IsTargetSet() {
	if (currentlinkinfo == NULL) {return -1;}
	if (currentlinkinfo->targetlist != NULL) {return 0;}
	return -1;
}
int linkinfo_IsBindSet() {
	if (currentlinkinfo == NULL) {return -1;}
	if (currentlinkinfo->bind != NULL) {return 0;}
	return -1;
}
void linkinfo_SetBind(const char* interface, int port, int iptype) {
	if (currentlinkinfo == NULL) {return;}
	currentlinkinfo->bind = malloc(strlen(interface)+1);
	if (currentlinkinfo->bind != NULL) {strcpy(currentlinkinfo->bind,interface);}
	currentlinkinfo->port = port;
	currentlinkinfo->iptype = iptype;
}
void linkinfo_SetTarget(const char* target) {
	if (currentlinkinfo == NULL) {return;}
	linkedlist_Append((void**)&currentlinkinfo->targetlist, target, strlen(target)+1);
}
int linkinfo_EndBlock() { /* end the new linking information block: check for missing values */
	if (linkinfo_IsProtocolSet() != 0) {
		cerr("Protocol for link block not set.", creader_GetLine());
		return -1;
	}
	if (linkinfo_IsBindSet() != 0) {
		cerr("Bind for link block not set.", creader_GetLine());
		return -1;
	}
	if (linkinfo_IsTargetSet() != 0) {
		cerr("TargetIP for link block not set.", creader_GetLine());
		return -1;
	}
	if (linkinfo_IsPasswordSet() != 0) {
		cerr("SendPassword for link block not set.", creader_GetLine());
		return -1;
	}
	if (linkinfo_IsRecvPasswordSet() != 0) {
		cerr("RecvPassword for link block not set.", creader_GetLine());
		return -1;
	}
	if (currentlinkinfo->iptype == IPTYPE_IPV6 && isipv4ip(linkedlist_GetData(currentlinkinfo->targetlist)) == 0) {
		cerr("Cannot specify IPv4 target for an IPv6-bound link.", creader_GetLine());
		return -1;
	}
	if (currentlinkinfo->iptype == IPTYPE_IPV4 && isipv6ip(linkedlist_GetData(currentlinkinfo->targetlist)) == 0) {
		cerr("Cannot specify IPv6 target for an IPv4-bound link.", creader_GetLine());
		return -1;
	}
	char pwstuf[100];
	snprintf(pwstuf,99,"%s %s",currentlinkinfo->password,currentlinkinfo->recvpassword);
	linkedlist_Append(&currentlinkinfo->passwordlist, pwstuf, strlen(pwstuf)+1);
	/* now find duplicates (important for passive links). attempt to merge or report error */
	void* item = linkinfoblocks;
	while (item != NULL) {
		struct linkinfo* i = linkedlist_GetData(item);
		if (i != currentlinkinfo) {
			if (i->port == currentlinkinfo->port && i->iptype == currentlinkinfo->iptype
			&& strcasecmp(i->bind,currentlinkinfo->bind) == 0) {
				char* p = linkedlist_GetData(currentlinkinfo->targetlist);
				char* login = linkedlist_GetData(currentlinkinfo->passwordlist);
				if (strcasecmp(i->protocol, currentlinkinfo->protocol) != 0) {
					cerr("Cannot mix multiple linking protocols at one listener port/interface.", creader_GetLine());
					return -1;
				}
				if (stringlist_CheckItem(&i->targetlist, p) == 0) {
					cerr("Link to this target already present.", creader_GetLine());
					return -1;
				}
				linkedlist_Append((void**)&i->targetlist, p, strlen(p)+1);
				linkedlist_Append((void**)&i->passwordlist, login, strlen(login)+1);
				linkinfo_ClearItemData(currentlinkinfo);
				linkedlist_RemoveItem(&linkinfoblocks,currentlinkinfo);
			}
		}
		item = linkedlist_NextItem(item);
	}
	//printf("block ended: %s:%d.\n", currentlinkinfo->interface,currentlinkinfo->port);
	currentlinkinfo = NULL;
	return 0;
}
void linkinfo_Apply(int isrehash) {
	/* first, terminate now unwanted links */
	
	/* then add the new ones if not present yet */
	void* item = linkinfoblocks;
	while (item != NULL) {
		struct linkinfo* l = linkedlist_GetData(item);
		//printf("applying %s:%d\n",l->bind,l->port);
		int ispresent = 0;
		void* passiveentry = NULL;
		void* item2 = listoflinks;
		while (item2 != NULL) {
			struct weircdlink* wl = linkedlist_GetData(item);
			if (strcasecmp(wl->interface, l->bind) == 0 && wl->port == l->port && wl->type == LINKTYPE_PASSIVE) {
				ispresent = 1;
				passiveentry = item;
				break;
			}
			item2 = linkedlist_NextItem(item2);
		}
		if (ispresent == 0) {
			/* add passive link */
			passiveentry = links_Add(0, l->bind, l->port, l->iptype, l->protocol, NULL, NULL, NULL);
			/* activate passive links right away */
			int r = links_Activate(l->bind,l->port);
			char msg[500];
			if (r < 0) {
				snprintf(msg,499,"%s LINKS: Failed to open passive link at %s:%d",startupprefix,l->bind,l->port);
				printf("%s\n",msg);
				IOnotice(msg);
			}else{
				snprintf(msg,499,"%s LINKS: Opened link listener at %s:%d",startupprefix,l->bind,l->port);
				printf("%s\n",msg);
				IOnotice(msg);
			}
		}
		if (passiveentry == NULL) {//failed to add a passive link. don't add active links
			item = linkedlist_NextItem(item);return;
		}
		/* add active link for each target host if not present yet */
		void* targetitem = l->targetlist;
		void* passwitem = l->passwordlist;
		while (targetitem != NULL) {
			ispresent = 0;
			item2 = listoflinks;
			while (item2 != NULL) {
				struct weircdlink* wl = linkedlist_GetData(item);
				if (strcasecmp(wl->interface, linkedlist_GetData(targetitem)) == 0 && wl->port == l->port
					&& wl->type == LINKTYPE_ACTIVE) {
					ispresent = 1;
					break;
				}
				item2 = linkedlist_NextItem(item2);
			}
			if (ispresent == 0) {
				char pw1[50];
				char pw2[150];
				char* p = linkedlist_GetData(passwitem);
				if (charpos(p, ' ') > 0) {
					strcopy(pw1,p,49);cutonfirstspace(pw1);
					strcopy(pw2,p,149);cut(pw2,charpos(pw2,' ')+1);
					links_Add(1, linkedlist_GetData(targetitem), l->port, l->iptype, l->protocol, pw1, pw2, passiveentry);
				}
				
				/*
				// activate the active link
				int r = links_Activate(l->interface,l->port);
				if (r < 0) {
					if (isrehash == 0) {
						printf("%s LINKS: Failed to open passive link at %s:%d\n",startupprefix,l->interface,l->port);
					}else{
						
					}
				}*/
			}
			targetitem = linkedlist_NextItem(targetitem);
			if (passwitem != NULL) {
				passwitem = linkedlist_NextItem(passwitem);
			}
		}
		item = linkedlist_NextItem(item);
	}
	//printf("%s Server links processed.\n",startupprefix);
}