//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----- This is the WeIRCd API for sockets  ----- //
  /* This can be used by any module to maintain internet
     connections.
     
     This api is:
       - designed for tcp/ip
       - able to deal with IPv6 (given WeIRCd is compiled with
         IPv6 support which is default for the Windows version)
       - allows you to use WeIRCd's select() management
         (be careful! you can easily push the server to an
          accidential 100% cpu polling loop here!)
       - is therefore also suitable for single-threaded
         connection management
         
     */
  
#ifndef INET6_ADDRSTRLEN
#define INET6_ADDRSTRLEN 46 //needed for Microsoft Windows
#endif

#define IPV4LEN 16

#ifdef IPV6
	#define IPV6LEN INET6_ADDRSTRLEN
	#define IPMAXLEN INET6_ADDRSTRLEN
#else
	#define IPMAXLEN 16
#endif



#define IPTYPE_IPV4 1
#define IPTYPE_IPV6 2

 /* PLEASE NOTE: Never use WeIRCd's select handling when you're not aware of how it works (basically
    you NEED to have a select() hook and when it gets called with a read and/or write alert you
    SHOULD be processing all the data or you WILL send the server into a 100% cpu loop!!) */
 
 /* PLEASE NOTE: Close all your socekts and resources properly on shutdown. WeIRCd won't (in most
    cases). */

int weircd_sockets_GetIPLen();
  /* The maximum len an ip address can possibly have in plain text representation. When IPv6
     is activated, this is the maximum len of an IPv6 address.
     When you want to use fixed size or pre-allocated buffers for storing ips, allocate the
     returned length value of this function + 1 (for null terminator) and you should be safe
     for all IPs WeIRCd ever passes to you as plain text representation. */
 
int weircd_sockets_CreateSocket(int addToSelect, int iptype);
  /* Create a new socket and specify the "iptype" (IPv4 or IPv6). Please note the latter is probably
     unavailable, which will result in -1 as a return value (also on any other error which is
     unlikely to happen though, but please handle it gracefully).
     "addToSelect" does, when set to 1, add the socket to WeIRCd's fdset. You can then (and you
     REALLY SHOULD or you will break things horribly when not taking care of this!!) hook into
     WeIRCd's select() processing. Please note you should be rather experienced with select()
     before attempting to do this - if you're not, better fork off your own internet connection
     thread that communicates with your main thread that handles the non-threadsafe WeIRCd api
     calls! */

void weircd_sockets_ManageSocketWithSelect(int socket);
  /* Put an already created socket into WeIRCd's select() management. See above what that means
     (do NOT use when you do not know what select() is or how it works!!) */

int weircd_sockets_MakeSocketListen(int socket, int port, int iptype, const char* bindip);
  /* Have a created socket listen at some port using the specified interface, the specified port and the
     specified interface type (IPv6 or IPv6). Please note you need to specify the right iptype in any case,
     even if the specified interface is clearly an ipv6 or ipv6 ip.
     Please note interface allows for "any" which should be used if you don't have a specific bind ip
     at hand. In that case, you MUST specify if you want an ipv4 or an ipv6 listener even more.
     Returns 0 on success and -1 on failure. */

void weircd_sockets_CloseSocket(int socket);
int weircd_sockets_AcceptConnection(int socket, int iptype, char* writeip, int* writesocket);
  /* Accept a connection at the socket. Returns 0 when that actually happened. Please do this
     check on all your listener sockets when you get a hooked select() handling call and there
     is any data to read reported.
     "socket" is the listener socket you plan to check and "iptype" is the iptype of that socket.
     Given the "writeip" buffer passed is not a NULL pointer, make sure it can keep at least
     weircd_sockets_GetIPLen()+1 bytes so you don't run into a segfault. Given the "writesocket"
     pointer is not NULL, WeIRCd will store the new socket there. */

int weircd_sockets_SelectSaysWrite(int sock);
int weircd_sockets_SelectSaysRead(int sock);
int weircd_sockets_SendOnSocket(int sock,const char* buffer,int buffsize);
int weircd_sockets_ReceiveData(int socket, char* buf, int len);

  /* Get familiar with those when you use WeIRCd's select handling: */
void weircd_sockets_IncreaseSend(); /* Add up using this if you plan to send stuff. Then WeIRCd will check for writeable
					sockets in its select() run. */
void weircd_sockets_DecreaseSend(); /* Substract when your stuff is sent (think of a reference counting mechanism).
					When you reach zero (below can't happen due to a minimum check and also
					shouldn't happen), WeIRCd will only check for incoming stuff and not
					pause on possibly writeable sockets.
					
				       NOTE: It is not necessary to restore the exact level of 0 before having
				        used weircd_sockets_IncreaseSend() on module shutdown. WeIRCd will
				        handle a remaining amount on module shutdown gracefully. */
void weircd_sockets_HookSelect();
  /* This will expect that your module provides the following function:
     void module_SelectHook(int readdata, int writedata);
     It will call the respective function and have readdata set to 1 given some sockets are
     probably having data on the read pipe and have writedata set to 1 when there are
     probably sockets having space on their write queue.
     Please note that you MUST NOT assume that this also is the case for any of your sockets.
     You MUST check all of them for this condition though and properly proceed with your
     workings if that's the case or you will send the IRCd into a 100% cpu loop when it
     always prompts this funtion again because the condition is still not solved. */

void weircd_sockets_UnhookSelect();
  /* Deactivate the select hook. */