//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// This implements modules/weircd_api_sockets.h functions and helper functions
// that are needed by the implementations.
// Additionally, it provides functions to call the select() hooks of modules
// using the socket api

#include "weircd.h"

struct modulesockinfo {
	char* name;
	int wantsend;
};
void* sockinfolist = NULL;

void* moduleselecthooklist = NULL;

void ModuleSockets_WipeModule(const char* modname) {
	void* ptr = modman_GetPtrByName(modname);
	/* remove our modulesockinfo entry that stores wantsend modifications */
	void* item = sockinfolist;
	while (item != NULL) {
		struct modulesockinfo* m = linkedlist_GetData(item);
		if (strcasecmp(m->name,modname) == 0) {
			wantsend -= m->wantsend; //revert any modifications
			free(m->name);
			linkedlist_RemoveItem(&sockinfolist,&item);
			break;
		}
		item = linkedlist_NextItem(item);
	}
	/* wipe us out of the hook list */
	item = moduleselecthooklist;
	while (item != NULL) {
		void* p = *((void**)linkedlist_GetData(item));
		if (p == ptr) {
			linkedlist_RemoveItem(&moduleselecthooklist,item);
			break;
		}
		item = linkedlist_NextItem(item);
	}
}
struct modulesockinfo* ModuleSockets_PickSockinfo(const char* modname) {
	void* item = sockinfolist;
	while (item != NULL) {
		struct modulesockinfo* m = linkedlist_GetData(item);
		if (strcasecmp(m->name,modname) == 0) {
			return m;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;	
}
struct modulesockinfo* ModuleSockets_Add(const char* modname) {
	struct modulesockinfo si;
	memset(&si,0,sizeof(si));
	si.name = malloc(strlen(modname)+1);
	if (si.name == NULL) {return NULL;}
	strcpy(si.name,modname);
	void* r = linkedlist_Append(&sockinfolist,&si,sizeof(si));
	if (r == NULL) {
		free(si.name);return NULL;
	}
	return linkedlist_GetData(r);
}
void weircd_sockets_IncreaseSend() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	struct modulesockinfo* s;
	s = ModuleSockets_PickSockinfo(module_Current());
	if (s == NULL) {
		s = ModuleSockets_Add(module_Current());
		if (s == NULL) {return;}
	}
	s->wantsend++;
	wantsend++;
}
void weircd_sockets_DecreaseSend() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	struct modulesockinfo* s;
	s = ModuleSockets_PickSockinfo(module_Current());
	if (s == NULL) {
		s = ModuleSockets_Add(module_Current());
		if (s == NULL) {return;}
	}
	if (s->wantsend > 0) {
		s->wantsend--;
		wantsend--;
	}
}
int weircd_sockets_CreateSocket(int addToSelect, int iptype) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_CreateSocket(addToSelect,iptype);
}
void weircd_sockets_CloseSocket(int socket) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;
	}
	so_CloseSocket(socket);
}
int weircd_sockets_AcceptConnection(int socket, int iptype, char* writeip, int* writesocket) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_AcceptConnection(socket,iptype,writeip,writesocket);
}
int weircd_sockets_SelectSaysWrite(int sock) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_SelectSaysWrite(sock);
}
int weircd_sockets_SelectSaysRead(int sock) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_SelectSaysRead(sock);
}
int weircd_sockets_SendOnSocket(int sock,const char* buffer,int buffsize) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_SendOnSocket(sock,buffer,buffsize);
}
int weircd_sockets_ReceiveData(int socket, char* buf, int len) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_ReceiveData(socket,buf,len);
}
int weircd_sockets_MakeSocketListen(int socket, int port, int iptype, const char* interface) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return so_MakeSocketListen(socket,port,iptype,interface);
}
void weircd_sockets_HookSelect() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	/* obtain module pointer to add to hook list */
	void* ptr = modman_GetPtrByName(module_Current());
	if (ptr == NULL) {return;}
	/* check whether we're already hooked */
	void* item = moduleselecthooklist;
	while (item != NULL) {
		void* p = *((void**)linkedlist_GetData(item));
		if (p == ptr) {return;} // we are already hooked -> quit
		item = linkedlist_NextItem(item);
	}
	/* add us to the list */
	linkedlist_Append(&moduleselecthooklist,&ptr,sizeof(void*));
}
void weircd_sockets_UnhookSelect() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	/* obtain module pointer to remove from hook list */
	void* ptr = modman_GetPtrByName(module_Current());
	if (ptr == NULL) {return;}
	/* find the list entry and remove it */
	void* item = moduleselecthooklist;
	while (item != NULL) {
		void* p = *((void**)linkedlist_GetData(item));
		if (p == ptr) {
			linkedlist_RemoveItem(&moduleselecthooklist,item);
			return;
		}
		item = linkedlist_NextItem(item);
	}
}
typedef void (*selecthookfunc)(int,int);
void weircd_sockets_ExecuteHooks(int read, int write) {
	void* item = moduleselecthooklist;
	while (item != NULL) {
		void* p = *((void**)linkedlist_GetData(item));
		if (p != NULL) {
			const char* n = modman_GetNameByPtr(p);
			if (n == NULL) {item = linkedlist_NextItem(item);continue;}
			module_Switched(n);module_SetArea(MODAREA_GENERICFUNC);
			selecthookfunc func = modman_GetDataByPtr(p,"module_SelectHook");
			if (func != NULL) {
				func(read,write);
			}
			module_Switched(NULL);
		}
		item = linkedlist_NextItem(item);
	}
}
